// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: BinaryOp.proto
// Protobuf C++ Version: 6.33.5

#ifndef BinaryOp_2eproto_2epb_2eh
#define BinaryOp_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6033005
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_BinaryOp_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_BinaryOp_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_BinaryOp_2eproto;
}  // extern "C"
namespace mojo {
namespace spec {
enum BinaryOpPosition : int;
extern const uint32_t BinaryOpPosition_internal_data_[];
enum BinaryOpType : int;
extern const uint32_t BinaryOpType_internal_data_[];
class BinaryOp;
struct BinaryOpDefaultTypeInternal;
extern BinaryOpDefaultTypeInternal _BinaryOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull BinaryOp_class_data_;
class ConstBinaryOp;
struct ConstBinaryOpDefaultTypeInternal;
extern ConstBinaryOpDefaultTypeInternal _ConstBinaryOp_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ConstBinaryOp_class_data_;
}  // namespace spec
}  // namespace mojo
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::mojo::spec::BinaryOpPosition_internal_data_>
    internal::EnumTraitsImpl::value<::mojo::spec::BinaryOpPosition>;
template <>
internal::EnumTraitsT<::mojo::spec::BinaryOpType_internal_data_>
    internal::EnumTraitsImpl::value<::mojo::spec::BinaryOpType>;
}  // namespace protobuf
}  // namespace google

namespace mojo {
namespace spec {
enum BinaryOpType : int {
  UNKNOWN_BINARY_OP_TYPE = 0,
  ADD = 1,
  SUBTRACT = 2,
  MULTIPLY = 3,
  DIVIDE = 4,
  POW = 5,
  MODULO = 6,
  BinaryOpType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  BinaryOpType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t BinaryOpType_internal_data_[];
inline constexpr BinaryOpType BinaryOpType_MIN =
    static_cast<BinaryOpType>(0);
inline constexpr BinaryOpType BinaryOpType_MAX =
    static_cast<BinaryOpType>(6);
inline bool BinaryOpType_IsValid(int value) {
  return 0 <= value && value <= 6;
}
inline constexpr int BinaryOpType_ARRAYSIZE = 6 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL BinaryOpType_descriptor();
template <typename T>
const ::std::string& BinaryOpType_Name(T value) {
  static_assert(::std::is_same<T, BinaryOpType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to BinaryOpType_Name().");
  return BinaryOpType_Name(static_cast<BinaryOpType>(value));
}
template <>
inline const ::std::string& BinaryOpType_Name(BinaryOpType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<BinaryOpType_descriptor, 0, 6>(
      static_cast<int>(value));
}
inline bool BinaryOpType_Parse(
    ::absl::string_view name, BinaryOpType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<BinaryOpType>(BinaryOpType_descriptor(), name,
                                           value);
}
enum BinaryOpPosition : int {
  UNKNOWN_BINARY_OP_POSITION = 0,
  LEFT = 1,
  RIGHT = 2,
  BinaryOpPosition_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  BinaryOpPosition_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t BinaryOpPosition_internal_data_[];
inline constexpr BinaryOpPosition BinaryOpPosition_MIN =
    static_cast<BinaryOpPosition>(0);
inline constexpr BinaryOpPosition BinaryOpPosition_MAX =
    static_cast<BinaryOpPosition>(2);
inline bool BinaryOpPosition_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int BinaryOpPosition_ARRAYSIZE = 2 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL BinaryOpPosition_descriptor();
template <typename T>
const ::std::string& BinaryOpPosition_Name(T value) {
  static_assert(::std::is_same<T, BinaryOpPosition>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to BinaryOpPosition_Name().");
  return BinaryOpPosition_Name(static_cast<BinaryOpPosition>(value));
}
template <>
inline const ::std::string& BinaryOpPosition_Name(BinaryOpPosition value) {
  return ::google::protobuf::internal::NameOfDenseEnum<BinaryOpPosition_descriptor, 0, 2>(
      static_cast<int>(value));
}
inline bool BinaryOpPosition_Parse(
    ::absl::string_view name, BinaryOpPosition* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<BinaryOpPosition>(BinaryOpPosition_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class ConstBinaryOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.ConstBinaryOp) */ {
 public:
  inline ConstBinaryOp() : ConstBinaryOp(nullptr) {}
  ~ConstBinaryOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ConstBinaryOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ConstBinaryOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ConstBinaryOp(::google::protobuf::internal::ConstantInitialized);

  inline ConstBinaryOp(const ConstBinaryOp& from) : ConstBinaryOp(nullptr, from) {}
  inline ConstBinaryOp(ConstBinaryOp&& from) noexcept
      : ConstBinaryOp(nullptr, ::std::move(from)) {}
  inline ConstBinaryOp& operator=(const ConstBinaryOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline ConstBinaryOp& operator=(ConstBinaryOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ConstBinaryOp& default_instance() {
    return *reinterpret_cast<const ConstBinaryOp*>(
        &_ConstBinaryOp_default_instance_);
  }
  enum ConstValCase {
    kInt32Val = 3,
    kInt64Val = 4,
    kFloat32Val = 5,
    kFloat64Val = 6,
    kBoolVal = 7,
    CONST_VAL_NOT_SET = 0,
  };
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(ConstBinaryOp& a, ConstBinaryOp& b) { a.Swap(&b); }
  inline void Swap(ConstBinaryOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ConstBinaryOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ConstBinaryOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ConstBinaryOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ConstBinaryOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ConstBinaryOp& from) { ConstBinaryOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ConstBinaryOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.ConstBinaryOp"; }

  explicit ConstBinaryOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ConstBinaryOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ConstBinaryOp& from);
  ConstBinaryOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ConstBinaryOp&& from) noexcept
      : ConstBinaryOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTypeFieldNumber = 1,
    kPosFieldNumber = 2,
    kInt32ValFieldNumber = 3,
    kInt64ValFieldNumber = 4,
    kFloat32ValFieldNumber = 5,
    kFloat64ValFieldNumber = 6,
    kBoolValFieldNumber = 7,
  };
  // .mojo.spec.BinaryOpType type = 1;
  void clear_type() ;
  ::mojo::spec::BinaryOpType type() const;
  void set_type(::mojo::spec::BinaryOpType value);

  private:
  ::mojo::spec::BinaryOpType _internal_type() const;
  void _internal_set_type(::mojo::spec::BinaryOpType value);

  public:
  // .mojo.spec.BinaryOpPosition pos = 2;
  void clear_pos() ;
  ::mojo::spec::BinaryOpPosition pos() const;
  void set_pos(::mojo::spec::BinaryOpPosition value);

  private:
  ::mojo::spec::BinaryOpPosition _internal_pos() const;
  void _internal_set_pos(::mojo::spec::BinaryOpPosition value);

  public:
  // int32 int32_val = 3;
  bool has_int32_val() const;
  void clear_int32_val() ;
  ::int32_t int32_val() const;
  void set_int32_val(::int32_t value);

  private:
  ::int32_t _internal_int32_val() const;
  void _internal_set_int32_val(::int32_t value);

  public:
  // int64 int64_val = 4;
  bool has_int64_val() const;
  void clear_int64_val() ;
  ::int64_t int64_val() const;
  void set_int64_val(::int64_t value);

  private:
  ::int64_t _internal_int64_val() const;
  void _internal_set_int64_val(::int64_t value);

  public:
  // float float32_val = 5;
  bool has_float32_val() const;
  void clear_float32_val() ;
  float float32_val() const;
  void set_float32_val(float value);

  private:
  float _internal_float32_val() const;
  void _internal_set_float32_val(float value);

  public:
  // double float64_val = 6;
  bool has_float64_val() const;
  void clear_float64_val() ;
  double float64_val() const;
  void set_float64_val(double value);

  private:
  double _internal_float64_val() const;
  void _internal_set_float64_val(double value);

  public:
  // bool bool_val = 7;
  bool has_bool_val() const;
  void clear_bool_val() ;
  bool bool_val() const;
  void set_bool_val(bool value);

  private:
  bool _internal_bool_val() const;
  void _internal_set_bool_val(bool value);

  public:
  void clear_const_val();
  ConstValCase const_val_case() const;
  // @@protoc_insertion_point(class_scope:mojo.spec.ConstBinaryOp)
 private:
  class _Internal;
  void set_has_int32_val();
  void set_has_int64_val();
  void set_has_float32_val();
  void set_has_float64_val();
  void set_has_bool_val();
  inline bool has_const_val() const;
  inline void clear_has_const_val();
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 7,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ConstBinaryOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int type_;
    int pos_;
    union ConstValUnion {
      constexpr ConstValUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      ::int32_t int32_val_;
      ::int64_t int64_val_;
      float float32_val_;
      double float64_val_;
      bool bool_val_;
    } const_val_;
    ::uint32_t _oneof_case_[1];
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_BinaryOp_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ConstBinaryOp_class_data_;
// -------------------------------------------------------------------

class BinaryOp final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:mojo.spec.BinaryOp) */ {
 public:
  inline BinaryOp() : BinaryOp(nullptr) {}
  ~BinaryOp() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(BinaryOp* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(BinaryOp));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR BinaryOp(::google::protobuf::internal::ConstantInitialized);

  inline BinaryOp(const BinaryOp& from) : BinaryOp(nullptr, from) {}
  inline BinaryOp(BinaryOp&& from) noexcept
      : BinaryOp(nullptr, ::std::move(from)) {}
  inline BinaryOp& operator=(const BinaryOp& from) {
    CopyFrom(from);
    return *this;
  }
  inline BinaryOp& operator=(BinaryOp&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const BinaryOp& default_instance() {
    return *reinterpret_cast<const BinaryOp*>(
        &_BinaryOp_default_instance_);
  }
  enum EpsilonCase {
    kFloat32Eps = 2,
    kFloat64Eps = 3,
    EPSILON_NOT_SET = 0,
  };
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(BinaryOp& a, BinaryOp& b) { a.Swap(&b); }
  inline void Swap(BinaryOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(BinaryOp* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  BinaryOp* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<BinaryOp>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const BinaryOp& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const BinaryOp& from) { BinaryOp::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(BinaryOp* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mojo.spec.BinaryOp"; }

  explicit BinaryOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  BinaryOp(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const BinaryOp& from);
  BinaryOp(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, BinaryOp&& from) noexcept
      : BinaryOp(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTypeFieldNumber = 1,
    kFloat32EpsFieldNumber = 2,
    kFloat64EpsFieldNumber = 3,
  };
  // .mojo.spec.BinaryOpType type = 1;
  void clear_type() ;
  ::mojo::spec::BinaryOpType type() const;
  void set_type(::mojo::spec::BinaryOpType value);

  private:
  ::mojo::spec::BinaryOpType _internal_type() const;
  void _internal_set_type(::mojo::spec::BinaryOpType value);

  public:
  // float float32_eps = 2;
  bool has_float32_eps() const;
  void clear_float32_eps() ;
  float float32_eps() const;
  void set_float32_eps(float value);

  private:
  float _internal_float32_eps() const;
  void _internal_set_float32_eps(float value);

  public:
  // double float64_eps = 3;
  bool has_float64_eps() const;
  void clear_float64_eps() ;
  double float64_eps() const;
  void set_float64_eps(double value);

  private:
  double _internal_float64_eps() const;
  void _internal_set_float64_eps(double value);

  public:
  void clear_epsilon();
  EpsilonCase epsilon_case() const;
  // @@protoc_insertion_point(class_scope:mojo.spec.BinaryOp)
 private:
  class _Internal;
  void set_has_float32_eps();
  void set_has_float64_eps();
  inline bool has_epsilon() const;
  inline void clear_has_epsilon();
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 3,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const BinaryOp& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int type_;
    union EpsilonUnion {
      constexpr EpsilonUnion() : _constinit_{} {}
      ::google::protobuf::internal::ConstantInitialized _constinit_;
      float float32_eps_;
      double float64_eps_;
    } epsilon_;
    ::uint32_t _oneof_case_[1];
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_BinaryOp_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull BinaryOp_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// ConstBinaryOp

// .mojo.spec.BinaryOpType type = 1;
inline void ConstBinaryOp::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::mojo::spec::BinaryOpType ConstBinaryOp::type() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ConstBinaryOp.type)
  return _internal_type();
}
inline void ConstBinaryOp::set_type(::mojo::spec::BinaryOpType value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mojo.spec.ConstBinaryOp.type)
}
inline ::mojo::spec::BinaryOpType ConstBinaryOp::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mojo::spec::BinaryOpType>(_impl_.type_);
}
inline void ConstBinaryOp::_internal_set_type(::mojo::spec::BinaryOpType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// .mojo.spec.BinaryOpPosition pos = 2;
inline void ConstBinaryOp::clear_pos() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pos_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::mojo::spec::BinaryOpPosition ConstBinaryOp::pos() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ConstBinaryOp.pos)
  return _internal_pos();
}
inline void ConstBinaryOp::set_pos(::mojo::spec::BinaryOpPosition value) {
  _internal_set_pos(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mojo.spec.ConstBinaryOp.pos)
}
inline ::mojo::spec::BinaryOpPosition ConstBinaryOp::_internal_pos() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mojo::spec::BinaryOpPosition>(_impl_.pos_);
}
inline void ConstBinaryOp::_internal_set_pos(::mojo::spec::BinaryOpPosition value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pos_ = value;
}

// int32 int32_val = 3;
inline bool ConstBinaryOp::has_int32_val() const {
  return const_val_case() == kInt32Val;
}
inline void ConstBinaryOp::set_has_int32_val() {
  _impl_._oneof_case_[0] = kInt32Val;
}
inline void ConstBinaryOp::clear_int32_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (const_val_case() == kInt32Val) {
    _impl_.const_val_.int32_val_ = 0;
    clear_has_const_val();
  }
}
inline ::int32_t ConstBinaryOp::int32_val() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ConstBinaryOp.int32_val)
  return _internal_int32_val();
}
inline void ConstBinaryOp::set_int32_val(::int32_t value) {
  if (const_val_case() != kInt32Val) {
    clear_const_val();
    set_has_int32_val();
  }
  _impl_.const_val_.int32_val_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ConstBinaryOp.int32_val)
}
inline ::int32_t ConstBinaryOp::_internal_int32_val() const {
  if (const_val_case() == kInt32Val) {
    return _impl_.const_val_.int32_val_;
  }
  return 0;
}

// int64 int64_val = 4;
inline bool ConstBinaryOp::has_int64_val() const {
  return const_val_case() == kInt64Val;
}
inline void ConstBinaryOp::set_has_int64_val() {
  _impl_._oneof_case_[0] = kInt64Val;
}
inline void ConstBinaryOp::clear_int64_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (const_val_case() == kInt64Val) {
    _impl_.const_val_.int64_val_ = ::int64_t{0};
    clear_has_const_val();
  }
}
inline ::int64_t ConstBinaryOp::int64_val() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ConstBinaryOp.int64_val)
  return _internal_int64_val();
}
inline void ConstBinaryOp::set_int64_val(::int64_t value) {
  if (const_val_case() != kInt64Val) {
    clear_const_val();
    set_has_int64_val();
  }
  _impl_.const_val_.int64_val_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ConstBinaryOp.int64_val)
}
inline ::int64_t ConstBinaryOp::_internal_int64_val() const {
  if (const_val_case() == kInt64Val) {
    return _impl_.const_val_.int64_val_;
  }
  return ::int64_t{0};
}

// float float32_val = 5;
inline bool ConstBinaryOp::has_float32_val() const {
  return const_val_case() == kFloat32Val;
}
inline void ConstBinaryOp::set_has_float32_val() {
  _impl_._oneof_case_[0] = kFloat32Val;
}
inline void ConstBinaryOp::clear_float32_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (const_val_case() == kFloat32Val) {
    _impl_.const_val_.float32_val_ = 0;
    clear_has_const_val();
  }
}
inline float ConstBinaryOp::float32_val() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ConstBinaryOp.float32_val)
  return _internal_float32_val();
}
inline void ConstBinaryOp::set_float32_val(float value) {
  if (const_val_case() != kFloat32Val) {
    clear_const_val();
    set_has_float32_val();
  }
  _impl_.const_val_.float32_val_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ConstBinaryOp.float32_val)
}
inline float ConstBinaryOp::_internal_float32_val() const {
  if (const_val_case() == kFloat32Val) {
    return _impl_.const_val_.float32_val_;
  }
  return 0;
}

// double float64_val = 6;
inline bool ConstBinaryOp::has_float64_val() const {
  return const_val_case() == kFloat64Val;
}
inline void ConstBinaryOp::set_has_float64_val() {
  _impl_._oneof_case_[0] = kFloat64Val;
}
inline void ConstBinaryOp::clear_float64_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (const_val_case() == kFloat64Val) {
    _impl_.const_val_.float64_val_ = 0;
    clear_has_const_val();
  }
}
inline double ConstBinaryOp::float64_val() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ConstBinaryOp.float64_val)
  return _internal_float64_val();
}
inline void ConstBinaryOp::set_float64_val(double value) {
  if (const_val_case() != kFloat64Val) {
    clear_const_val();
    set_has_float64_val();
  }
  _impl_.const_val_.float64_val_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ConstBinaryOp.float64_val)
}
inline double ConstBinaryOp::_internal_float64_val() const {
  if (const_val_case() == kFloat64Val) {
    return _impl_.const_val_.float64_val_;
  }
  return 0;
}

// bool bool_val = 7;
inline bool ConstBinaryOp::has_bool_val() const {
  return const_val_case() == kBoolVal;
}
inline void ConstBinaryOp::set_has_bool_val() {
  _impl_._oneof_case_[0] = kBoolVal;
}
inline void ConstBinaryOp::clear_bool_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (const_val_case() == kBoolVal) {
    _impl_.const_val_.bool_val_ = false;
    clear_has_const_val();
  }
}
inline bool ConstBinaryOp::bool_val() const {
  // @@protoc_insertion_point(field_get:mojo.spec.ConstBinaryOp.bool_val)
  return _internal_bool_val();
}
inline void ConstBinaryOp::set_bool_val(bool value) {
  if (const_val_case() != kBoolVal) {
    clear_const_val();
    set_has_bool_val();
  }
  _impl_.const_val_.bool_val_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.ConstBinaryOp.bool_val)
}
inline bool ConstBinaryOp::_internal_bool_val() const {
  if (const_val_case() == kBoolVal) {
    return _impl_.const_val_.bool_val_;
  }
  return false;
}

inline bool ConstBinaryOp::has_const_val() const {
  return const_val_case() != CONST_VAL_NOT_SET;
}
inline void ConstBinaryOp::clear_has_const_val() {
  _impl_._oneof_case_[0] = CONST_VAL_NOT_SET;
}
inline ConstBinaryOp::ConstValCase ConstBinaryOp::const_val_case() const {
  return ConstBinaryOp::ConstValCase(_impl_._oneof_case_[0]);
}
// -------------------------------------------------------------------

// BinaryOp

// .mojo.spec.BinaryOpType type = 1;
inline void BinaryOp::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::mojo::spec::BinaryOpType BinaryOp::type() const {
  // @@protoc_insertion_point(field_get:mojo.spec.BinaryOp.type)
  return _internal_type();
}
inline void BinaryOp::set_type(::mojo::spec::BinaryOpType value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mojo.spec.BinaryOp.type)
}
inline ::mojo::spec::BinaryOpType BinaryOp::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mojo::spec::BinaryOpType>(_impl_.type_);
}
inline void BinaryOp::_internal_set_type(::mojo::spec::BinaryOpType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// float float32_eps = 2;
inline bool BinaryOp::has_float32_eps() const {
  return epsilon_case() == kFloat32Eps;
}
inline void BinaryOp::set_has_float32_eps() {
  _impl_._oneof_case_[0] = kFloat32Eps;
}
inline void BinaryOp::clear_float32_eps() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (epsilon_case() == kFloat32Eps) {
    _impl_.epsilon_.float32_eps_ = 0;
    clear_has_epsilon();
  }
}
inline float BinaryOp::float32_eps() const {
  // @@protoc_insertion_point(field_get:mojo.spec.BinaryOp.float32_eps)
  return _internal_float32_eps();
}
inline void BinaryOp::set_float32_eps(float value) {
  if (epsilon_case() != kFloat32Eps) {
    clear_epsilon();
    set_has_float32_eps();
  }
  _impl_.epsilon_.float32_eps_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.BinaryOp.float32_eps)
}
inline float BinaryOp::_internal_float32_eps() const {
  if (epsilon_case() == kFloat32Eps) {
    return _impl_.epsilon_.float32_eps_;
  }
  return 0;
}

// double float64_eps = 3;
inline bool BinaryOp::has_float64_eps() const {
  return epsilon_case() == kFloat64Eps;
}
inline void BinaryOp::set_has_float64_eps() {
  _impl_._oneof_case_[0] = kFloat64Eps;
}
inline void BinaryOp::clear_float64_eps() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (epsilon_case() == kFloat64Eps) {
    _impl_.epsilon_.float64_eps_ = 0;
    clear_has_epsilon();
  }
}
inline double BinaryOp::float64_eps() const {
  // @@protoc_insertion_point(field_get:mojo.spec.BinaryOp.float64_eps)
  return _internal_float64_eps();
}
inline void BinaryOp::set_float64_eps(double value) {
  if (epsilon_case() != kFloat64Eps) {
    clear_epsilon();
    set_has_float64_eps();
  }
  _impl_.epsilon_.float64_eps_ = value;
  // @@protoc_insertion_point(field_set:mojo.spec.BinaryOp.float64_eps)
}
inline double BinaryOp::_internal_float64_eps() const {
  if (epsilon_case() == kFloat64Eps) {
    return _impl_.epsilon_.float64_eps_;
  }
  return 0;
}

inline bool BinaryOp::has_epsilon() const {
  return epsilon_case() != EPSILON_NOT_SET;
}
inline void BinaryOp::clear_has_epsilon() {
  _impl_._oneof_case_[0] = EPSILON_NOT_SET;
}
inline BinaryOp::EpsilonCase BinaryOp::epsilon_case() const {
  return BinaryOp::EpsilonCase(_impl_._oneof_case_[0]);
}
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace spec
}  // namespace mojo


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::mojo::spec::BinaryOpType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::mojo::spec::BinaryOpType>() {
  return ::mojo::spec::BinaryOpType_descriptor();
}
template <>
struct is_proto_enum<::mojo::spec::BinaryOpPosition> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::mojo::spec::BinaryOpPosition>() {
  return ::mojo::spec::BinaryOpPosition_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // BinaryOp_2eproto_2epb_2eh
